# -*- coding: utf-8 -*-
import maya.cmds as cmds
import maya.mel as mel

def color_index(value,hyper_panel_name):
    set_color_index = value + 1
    cmds.colorIndexSliderGrp(hyper_panel_name + 'hyp_color', edit=True,value=set_color_index)
    if cmds.ls( sl=True ):
        o_selected = (cmds.ls(selection = True,objectsOnly=True )[0])
        cmds.setAttr(o_selected + ".overrideEnabled", 1)
        cmds.setAttr(o_selected + ".overrideColor", value) 

def set_color_selected(hyper_panel_name):
    if not cmds.ls( sl=True ):
        print('Nothing is selected')
    else:
        first_selected = cmds.ls(sl=True)[0]
        selected_object = cmds.ls(sl=True)
        set_h_color_index =cmds.colorIndexSliderGrp(hyper_panel_name + 'hyp_color', q=True,value=True) - 1
        for node in selected_object:
            cmds.setAttr(node + ".overrideEnabled", 1)
            cmds.setAttr(node + ".overrideColor", set_h_color_index )
        cmds.select( clear=True )
        cmds.select( first_selected, replace=True )

def unfold_hierarchy_from_obj(hyper_panel_name):
    if not cmds.ls( sl=True ):
        print('Nothing is selected')
    else:
        o_selected = cmds.ls(sl=True)[0]
        mel.eval('string $o_selected = "%s"' % o_selected)
        robj = mel.eval('string $robj = `rootOf( $o_selected )`;' )
        cmds.select(robj,r=True)
        cmds.hyperGraph( hyper_panel_name + 'HyperGraphEd',edit=True,fold=False,unfoldAllShapes=True)
        cmds.select(o_selected,r=True)

def setsame_srt( o_same_srt ):
    if o_same_srt == 'Position':
        try:
            a_sel = cmds.ls( selection=True )[1]
            a_sel_w = cmds.xform(a_sel, worldSpace=True,q=True,translation=True)
            b_sel = cmds.ls( selection=True )[0]
            cmds.xform(b_sel, translation=a_sel_w,worldSpace=True,absolute=True )
        except:
            print("Select 2 node, to Move and to Where")
    if o_same_srt == 'Rotation':
        try:
            a_sel = cmds.ls( selection=True )[1]
            a_sel_w = cmds.xform(a_sel, worldSpace=True,q=True,rotation=True)
            b_sel = cmds.ls( selection=True )[0]
            cmds.xform(b_sel, rotation=a_sel_w,worldSpace=True,absolute=True )
        except:
            print("Select 2 node, to Rotate and to Which")
    if o_same_srt == 'Scale':
        try:
            a_sel = cmds.ls( selection=True )[1]
            a_sel_w = cmds.xform(a_sel, worldSpace=True,q=True,scale=True)
            b_sel = cmds.ls( selection=True )[0]
            cmds.xform(b_sel, scale=a_sel_w,worldSpace=True,absolute=True )
        except:
            print("Select 2 node, to Scale and to Which")

def const_types( o_const_types ):
    if o_const_types == 'Point':
        mel.eval('PointConstraintOptions;')
    elif o_const_types == 'Aim':
        mel.eval('AimConstraintOptions;')
    elif o_const_types == 'Orient':
        mel.eval('OrientConstraintOptions;')
    elif o_const_types == 'Scale':
        mel.eval('ScaleConstraintOptions;')
    elif o_const_types == 'Parent':
        mel.eval('ParentConstraintOptions;')
    elif o_const_types == 'Geometry':
        mel.eval('GeometryConstraintOptions;')
    elif o_const_types == 'Normal':
        mel.eval('NormalConstraintOptions;')
    elif o_const_types == 'PointOnPoly':
        mel.eval('PointOnPolyConstraintOptions;')
    elif o_const_types == 'ClosestPoint':
        mel.eval('ClosestPointOnOptions;')
    elif o_const_types == 'PoleVector':
        mel.eval('PoleVectorConstraintOptions;')
    elif o_const_types == 'RemoveConstraint':
        mel.eval('RemoveConstraintTargetOptions;')
    elif o_const_types == 'SetRestPosition':
        mel.eval('setConstraintRestPosition;')
    elif o_const_types == 'ModifyAxis':
        mel.eval('ModifyConstraintAxisOptions;')


def custom_hypergraph(*args):
    invis_panel = cmds.getPanel(invisiblePanels=True)

    hyphpanels = cmds.getPanel(scriptType='hyperGraphPanel')
    hyphpanels.remove('hyperGraphPanel1')

    for o_hyphpanel in hyphpanels:
        if o_hyphpanel in invis_panel:
            cmds.deleteUI(o_hyphpanel,panel=True)

    hyper_panel_no_a = str( len(cmds.getPanel( scriptType='hyperGraphPanel' )) + 1 )

    if cmds.window('custom_HyperGraph'+ hyper_panel_no_a, exists=True):
        n_graphpanels = cmds.getPanel(scriptType='hyperGraphPanel')
        biggest_p = 1
        for o_panel in n_graphpanels:
            if int( o_panel[len('hyperGraphPanel'):]) > biggest_p:
                biggest_p = int( o_panel[len('hyperGraphPanel'):]) 
        hyper_panel_no_a = str( biggest_p + 1 )

    try:
        win1 = cmds.window('custom_HyperGraph'+ hyper_panel_no_a, title='New CustomHyperGraph'+ hyper_panel_no_a,resizeToFitChildren=True, widthHeight=(800, 500))
    except:
        hyper_panel_no_a = str( int(hyper_panel_no_a) + 1 )
        win1 = cmds.window('custom_HyperGraph'+ hyper_panel_no_a, title='New CustomHyperGraph'+ hyper_panel_no_a,resizeToFitChildren=True, widthHeight=(800, 500))

    hyper_panel_name = "hyperGraphPanel" + hyper_panel_no_a

    pane1 = cmds.paneLayout(configuration='horizontal2', paneSize=[2,1,1], parent=win1)
    hypergraphmenu = 'hyperGraphPanel' + hyper_panel_no_a
    cmds.scriptedPanel(hypergraphmenu, label=hypergraphmenu ,type='hyperGraphPanel', parent=pane1)
    cmds.setParent('..')

    mel.eval('string $hie[] = `getPanel -sty "hyperGraphPanel"`;')
    mel.eval('string $schem = $hie[size($hie)-1] + "HyperGraphEd";')
    mel.eval('hyperGraph -edit -useDrawOverrideColor on $schem;')
    if cmds.optionVar( exists='chg_setup_var' ) == False:
        cmds.optionVar( stringValue=('chg_setup_var', '1122111' ) )

    load_chg_setup_var = cmds.optionVar( q='chg_setup_var' )

    if len(cmds.optionVar( q='chg_setup_var' )) == 6:
        old_load_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
        new_load_chg_setup_var = old_load_chg_setup_var[:6] + '1'
        cmds.optionVar( stringValue=('chg_setup_var', new_load_chg_setup_var ) )

    if load_chg_setup_var[:1]== '1':
        mel.eval('checkOrientCollection schematic $schem;')
    elif load_chg_setup_var[:1]== '2':
        mel.eval('checkOrientCollection horiz $schem;')
    elif load_chg_setup_var[:1]== '3':
        mel.eval('checkOrientCollection vert $schem;')
    if load_chg_setup_var[1:2]== '1':
        mel.eval('checkFreeformCollection On $schem;')
    elif load_chg_setup_var[1:2]== '2':
        mel.eval('checkFreeformCollection Off $schem;')
    if load_chg_setup_var[2:3]== '1':
        mel.eval('hyperGraph -edit -showShapes on $schem;')
    elif load_chg_setup_var[2:3]== '2':
        mel.eval('hyperGraph -edit -showShapes off $schem;')
    if load_chg_setup_var[3:4]== '1':
        mel.eval('hyperGraph -edit -showInvisible on $schem;')
    elif load_chg_setup_var[3:4]== '2':
        mel.eval('hyperGraph -edit -showInvisible off $schem;')
    if load_chg_setup_var[4:5]== '1':
        mel.eval('hyperGraph -edit -showExpressions on $schem;')
    elif load_chg_setup_var[4:5]== '2':
        mel.eval('hyperGraph -edit -showExpressions off $schem;')
    if load_chg_setup_var[5:6]== '1':
        mel.eval('hyperGraph -edit -showConstraints on $schem;')
    elif load_chg_setup_var[5:6]== '2':
        mel.eval('hyperGraph -edit -showConstraints off $schem;')
    if load_chg_setup_var[6:]== '1':
        mel.eval('hyperGraph -edit -showConnectionFromSelected off $schem;')
        mel.eval('hyperGraph -edit -showConnectionToSelected off $schem;')
    elif load_chg_setup_var[6:]== '2':
        mel.eval('hyperGraph -edit -showConnectionFromSelected on $schem;')
        mel.eval('hyperGraph -edit -showConnectionToSelected on $schem;')
    elif load_chg_setup_var[6:]== '3':
        mel.eval('hyperGraph -edit -showConnectionFromSelected off $schem;')
        mel.eval('hyperGraph -edit -showConnectionToSelected on $schem;')
    elif load_chg_setup_var[6:]== '4':
        mel.eval('hyperGraph -edit -showConnectionFromSelected on $schem;')
        mel.eval('hyperGraph -edit -showConnectionToSelected off $schem;')


    cmds.columnLayout( adjustableColumn=True )
    cmds.rowLayout(numberOfColumns=4)
    cmds.button(hyper_panel_name + 'select_hierarchy',label=' SelectHierarchy ',bgc=[0.3,0.3,0.3],command='cmds.select(hierarchy=True)')
    cmds.button(hyper_panel_name + 'unfold_hierarchy',label=' UnfoldHierarchy ',bgc=[0.35,0.35,0.35],
        command=lambda x:unfold_hierarchy_from_obj(hyper_panel_name))

    cmds.optionMenu(hyper_panel_name + 'set_srt', label='  SetSameSRT>', changeCommand=setsame_srt )
    cmds.menuItem( label='Position' )
    cmds.menuItem( label='Rotation' )
    cmds.menuItem( label='Scale' )

    cmds.optionMenu(hyper_panel_name + 'ConstraintOp', label='  ConstraintOp>',width=190, changeCommand=const_types )
    cmds.menuItem( label='Point' )
    cmds.menuItem( label='Aim' )
    cmds.menuItem( label='Orient' )
    cmds.menuItem( label='Scale' )
    cmds.menuItem( label='Parent' )
    cmds.menuItem( label='Geometry' )
    cmds.menuItem( label='Normal' )
    cmds.menuItem( label='PointOnPoly' )
    cmds.menuItem( label='ClosestPoint' )
    cmds.menuItem( label='PoleVector' )
    cmds.menuItem( label='RemoveConstraint' )
    cmds.menuItem( label='SetRestPosition' )
    cmds.menuItem( label='ModifyAxis' )
    cmds.setParent('..')
    cmds.rowLayout(numberOfColumns=4)
    cmds.button(hyper_panel_name + 'connection_editor',label='  ConnectionEd  ',bgc=[0.2,0.2,0.2],command="mel.eval('ConnectionEditor;')")
    cmds.button(hyper_panel_name + 'expression_editor',label='   ExpressionEd   ',bgc=[0.2,0.2,0.2],command="mel.eval('ExpressionEditor;')")
    cmds.button(hyper_panel_name + 'search_replace_name',label=' ReplaceName ',bgc=[0.3,0.3,0.3],command="mel.eval('performSearchReplaceNames 1;')")
    cmds.button(hyper_panel_name + 'namespac_enditor',label=' NamespaceEd ',command="mel.eval('NamespaceEditor;')")
    cmds.setParent('..')
    cmds.rowLayout(numberOfColumns=5)
    cmds.button('chg_setuo',label=' Setup ',height=13,width=53,bgc=[0.1,0.1,0.1],annotation='Open_SetupMenu',command=chg_setup_menu)
    cmds.text('  ')
    cmds.colorIndexSliderGrp(hyper_panel_name + 'hyp_color',label='Select Color ',min=0,max=32,value=0,columnWidth3=[77,70,0])
    cmds.intSlider(hyper_panel_no_a + 'hyp_color_index', min=0, max=31, value=0, step=1,dragCommand=lambda x:color_index(int(x),hyper_panel_name))
    cmds.button(label='SetColorToSelected',command=lambda x:set_color_selected(hyper_panel_name))

    cmds.showWindow()

def chg_setup_menu(*args):
    mel.eval('string $hie[] = `getPanel -sty "hyperGraphPanel"`;')
    mel.eval('string $schem = $hie[size($hie)-1] + "HyperGraphEd";')

    def radio_button_hierarchy(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = '1' + old_chg_setup_var[1:]
            mel.eval('checkOrientCollection schematic $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = '2' + old_chg_setup_var[1:]
            mel.eval('checkOrientCollection horiz $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 3:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = '3' + old_chg_setup_var[1:]
            mel.eval('checkOrientCollection vert $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )

    def radio_button_layout(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:1] + '1' + old_chg_setup_var[2:]
            mel.eval('checkFreeformCollection On $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:1] + '2' + old_chg_setup_var[2:]
            mel.eval('checkFreeformCollection Off $schem;')	
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )

    def radio_button_shape(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:2] + '1' + old_chg_setup_var[3:]
            mel.eval('hyperGraph -edit -showShapes on $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:2] + '2' + old_chg_setup_var[3:]
            mel.eval('hyperGraph -edit -showShapes off $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )

    def radio_button_hide(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:3] + '1' + old_chg_setup_var[4:]
            mel.eval('hyperGraph -edit -showInvisible on $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:3] + '2' + old_chg_setup_var[4:]
            mel.eval('hyperGraph -edit -showInvisible off $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )

    def radio_button_expression(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:4] + '1' + old_chg_setup_var[5:]
            mel.eval('hyperGraph -edit -showExpressions on $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:4] + '2' + old_chg_setup_var[5:]
            mel.eval('hyperGraph -edit -showExpressions off $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )

    def radio_button_constraint(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:5] + '1' + old_chg_setup_var[6:]
            mel.eval('hyperGraph -edit -showConstraints on $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:5] + '2' + old_chg_setup_var[6:]
            mel.eval('hyperGraph -edit -showConstraints off $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )

    def showconnect(value):
        if value == 1:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:6] + '1'
            mel.eval('hyperGraph -edit -showConnectionFromSelected off $schem;')
            mel.eval('hyperGraph -edit -showConnectionToSelected off $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 2:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:6] + '2'
            mel.eval('hyperGraph -edit -showConnectionFromSelected on $schem;')
            mel.eval('hyperGraph -edit -showConnectionToSelected on $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 3:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:6] + '4'
            mel.eval('hyperGraph -edit -showConnectionFromSelected off $schem;')
            mel.eval('hyperGraph -edit -showConnectionToSelected on $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
        elif value == 4:
            old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
            new_chg_setup_var = old_chg_setup_var[:6] + '3'
            mel.eval('hyperGraph -edit -showConnectionFromSelected on $schem;')
            mel.eval('hyperGraph -edit -showConnectionToSelected off $schem;')
            cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )


    if cmds.window('custom_hypergraph_setup', exists=True):
        cmds.deleteUI('custom_hypergraph_setup')

    if cmds.optionVar( exists='chg_setup_var' ) == False:
        cmds.optionVar( stringValue=('chg_setup_var', '1122111' ) )
    if len(cmds.optionVar( q='chg_setup_var' )) == 6:
        old_chg_setup_var = cmds.optionVar( q='chg_setup_var' )
        new_chg_setup_var = old_chg_setup_var[:6] + '1'
        cmds.optionVar( stringValue=('chg_setup_var', new_chg_setup_var ) )
    else:
        exist_chg_setup_var = cmds.optionVar( q='chg_setup_var' )

        cmds.window('custom_hypergraph_setup',title='Display Setup',
            sizeable=True, menuBar=True, topLeftCorner=[220, 200], widthHeight=(321, 320))
        cmds.menu( label='File' )
        cmds.menuItem( label='exit', command="cmds.deleteUI( 'custom_hypergraph_setup' )" )

        cmds.scrollLayout( 'scrollLayout' )
        cmds.columnLayout( adjustableColumn=True )
        cmds.text(label='Custom HyperGraph Display Setup',bgc=[0.2,0.2,0.2],align='center' )
        cmds.separator( height=15, style='in' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('Hierarchy  ')
        cmds.radioButtonGrp(labelArray3=['Schematic', 'Horizontal', 'Vertical'],numberOfRadioButtons=3,columnWidth3=[80,80,60],
            select=int(exist_chg_setup_var[:1]),
            onCommand1=lambda x:radio_button_hierarchy(1),
            onCommand2=lambda x:radio_button_hierarchy(2),
            onCommand3=lambda x:radio_button_hierarchy(3))
        cmds.showWindow()
        
        cmds.setParent( '..' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('Layout     ')
        cmds.radioButtonGrp(labelArray2=['Free', 'Auto'],numberOfRadioButtons=2,columnWidth2=[70,50],
            select=int(exist_chg_setup_var[1:2]),
            onCommand1=lambda x:radio_button_layout(1),
            onCommand2=lambda x:radio_button_layout(2))
        
        cmds.setParent( '..' )
        cmds.separator( height=15, style='out' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('Display     Shape Node    ')
        cmds.radioButtonGrp(labelArray2=['Yes', 'No'],numberOfRadioButtons=2,columnWidth2=[50,50],
            select=int(exist_chg_setup_var[2:3]),
            onCommand1=lambda x:radio_button_shape(1),
            onCommand2=lambda x:radio_button_shape(2))

        cmds.setParent( '..' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('              Hide Node      ')
        cmds.radioButtonGrp(labelArray2=['Yes', 'No'],numberOfRadioButtons=2,columnWidth2=[50,50],
            select=int(exist_chg_setup_var[3:4]),
            onCommand1=lambda x:radio_button_hide(1),
            onCommand2=lambda x:radio_button_hide(2))

        cmds.setParent( '..' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('              Expression      ')
        cmds.radioButtonGrp(labelArray2=['Yes', 'No'],numberOfRadioButtons=2,columnWidth2=[50,50],
            select=int(exist_chg_setup_var[4:5]),
            onCommand1=lambda x:radio_button_expression(1),
            onCommand2=lambda x:radio_button_expression(2))

        cmds.setParent( '..' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('              Constraint      ')
        cmds.radioButtonGrp(labelArray2=['Yes', 'No'],numberOfRadioButtons=2,columnWidth2=[50,50],
            select=int(exist_chg_setup_var[5:6]),
            onCommand1=lambda x:radio_button_constraint(1),
            onCommand2=lambda x:radio_button_constraint(2))

        cmds.setParent( '..' )
        cmds.separator( height=25, style='out' )

        cmds.rowLayout(numberOfColumns=1)
        cmds.text(' Filter Show Connection')
        cmds.setParent( '..' )
        cmds.rowLayout(numberOfColumns=2)
        cmds.text('To/FromSelected')
        cmds.radioButtonGrp(labelArray4=['NoFilter','To/From', 'To', 'From'],numberOfRadioButtons=4,columnWidth4=[68,73,35,40],
            select=int(exist_chg_setup_var[6:]),
            onCommand1=lambda x:showconnect(1),
            onCommand2=lambda x:showconnect(2),
            onCommand3=lambda x:showconnect(3),
            onCommand4=lambda x:showconnect(4))

        cmds.setParent( '..' )
        cmds.separator( height=25, style='in' )
        cmds.button(label='Close Settings', command="cmds.deleteUI( 'custom_hypergraph_setup' )" )

        cmds.showWindow()
